CreateServerScreen {
    border: round white;
    width: 5;
    height: 5;
}

Screen {
    padding-left: 1;
    background: black;
}

Label {
    height: 1;
    padding: 0 1;
    margin: 0 0 1 0;
}

.button {
    margin: 0 1 1 0;
    padding: 0;
    min-width: 17;
    border: none;
}

.log_window {
    height: auto;
    min-height: 5;
    max-height: 30;
    width: 90%;
}

#ready_configs {
    height: 4;
}

#config_horizontal {
    border: round rgb(0, 98, 255);
}

#running_screen_log_window {
    height: auto;
    min-height: 5;
    width: auto;
    min-width: 50%;
}

#main_log_window {
    min-height: 8;
}

Vertical,
Horizontal {
    height: auto;
}

RadioButton {
    border: none;
    padding: 0 2;
}

Input {
    width: 85%;
    height: auto;
    padding: 0 2;
    border: none;
}

Checkbox {
    border: none;
    width: 95%;
    padding: 0;
}

Collapsible {
    border: none;
    padding: 0;
    padding-bottom: 1;
}

DataTable {
    width: auto;
}